using System;
using System.Drawing;
using System.Windows.Forms;
using System.Diagnostics;
using System.Collections;
using System.ComponentModel;
using System.ComponentModel.Design;
using Infragistics.Win.CalcEngine;

namespace Infragistics.Win.UltraWinCalcManager.FormulaBuilder
{
	#region Event Handlers (delegates)

		#region OperandInitializingEventHandler

	/// <summary>
	/// Delegate for handling the <see cref="FormulaBuilderDialog.OperandInitializing"/> event.
	/// </summary>
	public delegate void OperandInitializingEventHandler(object sender, OperandInitializingEventArgs e); 

		#endregion //OperandInitializingEventHandler

		#region FunctionInitializingEventHandler

	/// <summary>
	/// Delegate for handling the <see cref="FormulaBuilderDialog.FunctionInitializing"/> event.
	/// </summary>
	public delegate void FunctionInitializingEventHandler(object sender, FunctionInitializingEventArgs e); 

		#endregion //FunctionInitializingEventHandler

	#endregion Event Handlers (delegates)

	#region Event Args

	#region OperandInitializingEventArgs

	/// <summary>
	/// Event parameters used for the <see cref="FormulaBuilderDialog.OperandInitializing"/> event.
	/// </summary>
	/// <seealso cref="OperandInitializingEventHandler"/>
	public class OperandInitializingEventArgs : CancelEventArgs
	{
		#region Member Variables

		private string operandName = null;
		private object operandContext = null;
		private IFormulaProvider formulaProvider = null;
        // LZ-GSG 03/14/2013
        private string operandAlias = null;

		#endregion Member Variables

		#region Constructor

		/// <summary>
		/// Constructor
		/// </summary>
        /// <param name="formulaProvider">An <see cref="IFormulaProvider"/> which provides formula information to the dialog.</param>
        /// <param name="operandContext">The context of the Reference.</param>
        /// <param name="operandName">The actual name of the reference which is being initialized.</param>
		public OperandInitializingEventArgs(string operandName, object operandContext, IFormulaProvider formulaProvider)
		{
			this.operandName		= operandName;
			this.operandContext		= operandContext;
			this.formulaProvider	= formulaProvider;
		}

       
        // LZ-GSG 03/14/2013
        public OperandInitializingEventArgs(string operandName, object operandContext, string operandAlias, IFormulaProvider formulaProvider)
        {
            this.operandName = operandName;
            this.operandContext = operandContext;
            this.formulaProvider = formulaProvider;
            this.operandAlias = operandAlias;
        }

		#endregion Constructor

		#region Properties

		#region OperandName

		/// <summary>
		/// Read-only property that returns the actual name of the reference which is being initialized.
		/// </summary>
		public string OperandName
		{
			get	{ return this.operandName; }
		}

		#endregion OperandName

		#region OperandContext

		/// <summary>
		/// Read-only property that returns the context of the Reference. For example, this could be an UltraGridColumn, a SummaryValue, a TextBox, etc. 
		/// </summary>
		public object OperandContext
		{
			get	{ return this.operandContext; }
		}

		#endregion OperandContext

		#region FormulaProvider

		/// <summary>
		/// Read-only property that returns the IFormulaProvider. The IFormulaProvider is the object whose format property is being edited.
		/// </summary>
		public IFormulaProvider FormulaProvider
		{
			get	{ return this.formulaProvider; }
		}

		#endregion FormulaProvider

		#region OperandAlias

//		/// <summary>
//		/// An alias which will be used for this reference for the duration of the Formula Builder Dialog session. 
//		/// </summary>
//		/// <remarks>
//		/// <p class="body">The OperandAlias can be used to provide a user-friendly display name for an operand in the Formula Builder dialog. For example, a reference which is the name of a control like "textBox1" could be changed to something more user-friendly, like "Radius". This alias applies only to the Formula Builder dialog. The Formula Property of the FormulaBuilder dialog will return the formula using the real reference names. The returned alias, should be only the name of the current reference and should not include the complete path information like the FullName.</p>
//		/// </remarks>
        
        // LZ-GSG 03/14/2013
        public string OperandAlias
        {
            get { return this.operandAlias; }
            set { this.operandAlias = value; }
        }

		#endregion OperandAlias

		#endregion Properties
	}

	#endregion //OperandInitializingEventArgs

	#region FunctionInitializingEventArgs

	/// <summary>
	/// Event parameters used for the <see cref="FormulaBuilderDialog.FunctionInitializing"/> event.
	/// </summary>
	/// <seealso cref="FunctionInitializingEventHandler"/>
	public class FunctionInitializingEventArgs : CancelEventArgs
	{
		#region Member Variables

		private UltraCalcFunction function = null;		
		private IFormulaProvider formulaProvider = null;

		#endregion Member Variables

		#region Constructor

		/// <summary>
		/// Constructor
		/// </summary>        
        /// <param name="function">The function being initialized.</param>
        /// <param name="formulaProvider">An <see cref="IFormulaProvider"/> which provides formula information to the dialog.</param>
		public FunctionInitializingEventArgs(UltraCalcFunction function, IFormulaProvider formulaProvider)
		{
			this.function    = function;
			this.formulaProvider = formulaProvider;
		}

		#endregion Constructor

		#region Properties

		#region FunctionRealName

		/// <summary>
		/// Read-only property that returns the actual name of the reference which is being initialized.
		/// </summary>
		public UltraCalcFunction Function
		{
			get	{ return this.function; }
		}

		#endregion FunctionName

		#region FormulaProvider

		/// <summary>
		/// Read-only property that returns the IFormulaProvider. The IFormulaProvider is the object whose format property is being edited.
		/// </summary>
		public IFormulaProvider FormulaProvider
		{
			get	{ return this.formulaProvider; }
		}

		#endregion FormulaProvider

		#endregion Properties
	}

	#endregion //FunctionInitializingEventArgs

	#endregion Event Args
}

#region Copyright (c) 2001-2012 Infragistics, Inc. All Rights Reserved
/* ---------------------------------------------------------------------*
*                           Infragistics, Inc.                          *
*              Copyright (c) 2001-2012 All Rights reserved               *
*                                                                       *
*                                                                       *
* This file and its contents are protected by United States and         *
* International copyright laws.  Unauthorized reproduction and/or       *
* distribution of all or any portion of the code contained herein       *
* is strictly prohibited and will result in severe civil and criminal   *
* penalties.  Any violations of this copyright will be prosecuted       *
* to the fullest extent possible under law.                             *
*                                                                       *
* THE SOURCE CODE CONTAINED HEREIN AND IN RELATED FILES IS PROVIDED     *
* TO THE REGISTERED DEVELOPER FOR THE PURPOSES OF EDUCATION AND         *
* TROUBLESHOOTING. UNDER NO CIRCUMSTANCES MAY ANY PORTION OF THE SOURCE *
* CODE BE DISTRIBUTED, DISCLOSED OR OTHERWISE MADE AVAILABLE TO ANY     *
* THIRD PARTY WITHOUT THE EXPRESS WRITTEN CONSENT OF INFRAGISTICS, INC. *
*                                                                       *
* UNDER NO CIRCUMSTANCES MAY THE SOURCE CODE BE USED IN WHOLE OR IN     *
* PART, AS THE BASIS FOR CREATING A PRODUCT THAT PROVIDES THE SAME, OR  *
* SUBSTANTIALLY THE SAME, FUNCTIONALITY AS ANY INFRAGISTICS PRODUCT.    *
*                                                                       *
* THE REGISTERED DEVELOPER ACKNOWLEDGES THAT THIS SOURCE CODE           *
* CONTAINS VALUABLE AND PROPRIETARY TRADE SECRETS OF INFRAGISTICS,      *
* INC.  THE REGISTERED DEVELOPER AGREES TO EXPEND EVERY EFFORT TO       *
* INSURE ITS CONFIDENTIALITY.                                           *
*                                                                       *
* THE END USER LICENSE AGREEMENT (EULA) ACCOMPANYING THE PRODUCT        *
* PERMITS THE REGISTERED DEVELOPER TO REDISTRIBUTE THE PRODUCT IN       *
* EXECUTABLE FORM ONLY IN SUPPORT OF APPLICATIONS WRITTEN USING         *
* THE PRODUCT.  IT DOES NOT PROVIDE ANY RIGHTS REGARDING THE            *
* SOURCE CODE CONTAINED HEREIN.                                         *
*                                                                       *
* THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE.              *
* --------------------------------------------------------------------- *
*/
#endregion Copyright (c) 2001-2012 Infragistics, Inc. All Rights Reserved