using System;
using System.Drawing.Design;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using Infragistics.Win.CalcEngine;
using System.Diagnostics;
using System.Drawing;
using Infragistics.Win.UltraWinTree;
using System.Collections;
using Infragistics.Win.UltraWinToolbars;
using Infragistics.Shared;
using Infragistics.Win.UltraWinCalcManager;


namespace Infragistics.Win.UltraWinCalcManager.FormulaBuilder
{
	/// <summary>
	/// Summary description for FormulaUITypeEditor.
	/// </summary>
	public class FormulaUITypeEditor : UITypeEditor
	{
		#region Constructor
		/// <summary>
		/// FormulaUITypeEditor Construtor
		/// </summary>
		public FormulaUITypeEditor()
		{
			
		}
		#endregion Constructor

		#region Private / Internal Members
		
		internal IFormulaBuilderInfo designInfo = null;
		internal IFormulaProvider formulaProvider = null;
		internal FormulaBuilderDialog formulaBuilder = null;

		internal static readonly string	DESIGNER_REG_KEY = "Software\\Infragistics\\UltraWinCalcManager\\DesignerDialog";

		static internal TreeStateManager formulaBuilderOperandsTreeStateManager = null;
		static internal TreeStateManager formulaBuilderFunctionTreeStateManager = null;
		static internal TreeStateManager functionBuilderOperandsTreeStateManager = null;

		#endregion Private Members
        
		#region Constants
		internal static readonly string FunctionAllCategoryName = SR.GetString("FunctionCategories_All");
		internal static readonly string FunctionUnassignedCategoryName = SR.GetString("FunctionCategories_Unassigned");
		#endregion Constants

		#region Methods

		#region Private / Internal Methods
		
		#region PopulateOperandsTree

		internal static void PopulateOperandsTree(UltraTree tree, IFormulaBuilderInfo designInfo, IFormulaProvider formulaProvider, AliasManager aliasManager)
		{
			// MRS 1/17/05 - Added support for aliasing/cancelling references
			//ReferenceNode[] refNodes = designInfo.GetReferenceTree();
            
            // LZ-GSG 03/14/2013
            ReferenceNode[] refNodes = aliasManager.refTree;
            

			tree.NodeLevelOverrides[0].Sort = SortType.None;
			tree.Override.Sort = SortType.Ascending;
			tree.Override.SelectionType = SelectType.SingleAutoDrag;

			tree.Nodes.Clear();

            // MBS 12/21/06
            // Added support for conditional formatting to show the node at the top of the tree
            if (formulaProvider is IFormulaProviderEx)
            {
                ReferenceNode node = ((IFormulaProviderEx)formulaProvider).FormulaDialogReferenceNode;
                if (node != null)
                    FormulaUITypeEditor.AddOperandNode(tree, node, formulaProvider, aliasManager);
            }

            foreach (ReferenceNode refNode in refNodes)
            {
                FormulaUITypeEditor.AddOperandNode(tree, refNode, formulaProvider, aliasManager);
                //SetOperandTreeAlias(refNode, tree, formulaProvider, aliasManager);
            }
		}


        internal static void SetOperandTreeAlias(ReferenceNode parentNode, UltraTree tree, IFormulaProvider formulaProvider, AliasManager aliasManager)
        {
            foreach (ReferenceNode node in parentNode.ChildReferences)
            {
                FormulaUITypeEditor.AddOperandNode(tree, node, formulaProvider, aliasManager);
                if (node.ChildReferences != null) SetOperandTreeAlias(node, tree, formulaProvider, aliasManager);
                else return;
            }
        }

		#endregion PopulateOperandsTree

		#region AddOperandNode

		internal static void AddOperandNode(UltraTree tree, ReferenceNode refNode, IFormulaProvider formulaProvider, AliasManager aliasManager)
		{

			//MRS 1/12/05 - Added support for aliasing/cancelling references.
			//if ( aliasManager.IsExcluded(refNode) )
				//return;
            
            // LZ-GSG 03/14/2013
			UltraTreeNode treeNode =null;

			string refDisplayName = GetReferenceDisplayName(refNode, aliasManager);

			if ( refNode.IsDataReference &&  refNode.Reference != formulaProvider.Reference)
			{
				string relativeName = FormulaUITypeEditor.GetRelativeReferenceName(formulaProvider.Reference, refNode.Reference);
                if (refDisplayName == null) refDisplayName = relativeName;
				ReferenceTreeNode refTreeNode = new ReferenceTreeNode(refDisplayName, refNode.Reference, relativeName);
				treeNode = (UltraTreeNode)refTreeNode;
				tree.Nodes.Add(treeNode);
			}
			else
			{
				// MRS 1/17/05 - Added support for aliasing/cancelling references
				//treeNode = tree.Nodes.Add("", refDisplayName);	
				//if (refNode.Reference != null 
				//	|| !aliasManager.AreExcluded(refNode.ChildReferences))
				//{
                
                // LZ-GSG 03/14/2013
                if (refDisplayName == null)
                {
                    treeNode = tree.Nodes.Add(refNode.DisplayNameResolved, refNode.DisplayNameResolved);
                }
                else
                {
                    treeNode = tree.Nodes.Add(refNode.DisplayNameResolved, refDisplayName);
                }
				//}
			}

			// MRS 1/17/05 - If a category node has no children
			// or all of it's children are excluded, then there will be no
			// node here. So bail. 
			if (treeNode == null) return;

			if ( refNode.IsDataReference &&  refNode.Reference != null )
			{
				treeNode.Override.NodeAppearance.FontData.Bold = DefaultableBoolean.True;
			}			

			treeNode.Override.NodeAppearance.Image = FormulaUITypeEditor.GetOperandNodeImage(refNode);			
			
			// MRS BR00802
			// Moved down. Only do this if there are children
//			if ( refNode.Reference == formulaProvider.Reference )
//				treeNode.Override.NodeAppearance.ForeColor = SystemColors.GrayText;

			ReferenceNode[] refNodes = refNode.ChildReferences;

			if ( refNodes != null && refNodes.Length > 0 )
			{
				// MRS BR00802
				// Moved from above. Only do this if there are children
				if ( refNode.Reference == formulaProvider.Reference )
					treeNode.Override.NodeAppearance.ForeColor = SystemColors.GrayText;

				foreach (ReferenceNode childRefNode in refNodes )
				{
					FormulaUITypeEditor.AddOperandNode(treeNode, childRefNode, formulaProvider, aliasManager);
				}
			}
			else
			{
				// MRS BR00802
				// If there are no children, disable the node
				if ( refNode.Reference == formulaProvider.Reference )
					treeNode.Enabled = false;
			}

		}	

		internal static void AddOperandNode(UltraTreeNode parentNode, ReferenceNode refNode, IFormulaProvider formulaProvider, AliasManager aliasManager)
		{
			//MRS 1/12/05 - Added support for aliasing/cancelling references.
		//	if ( aliasManager.IsExcluded(refNode) )
	///			return;

            // LZ-GSG 03/14/2013
			UltraTreeNode treeNode =null;

			string refDisplayName = GetReferenceDisplayName(refNode, aliasManager);

			if ( refNode.IsDataReference && 
				refNode.Reference != formulaProvider.Reference)
			{
				string relativeName = FormulaUITypeEditor.GetRelativeReferenceName(formulaProvider.Reference, refNode.Reference);

                // LZ-GSG 03/14/2013
                if (refDisplayName == null) refDisplayName = relativeName;
				ReferenceTreeNode refTreeNode = new ReferenceTreeNode(refDisplayName, refNode.Reference, relativeName);
				
				treeNode = (UltraTreeNode)refTreeNode;
				parentNode.Nodes.Add(treeNode);
			}
			else
			{
				// MRS 1/17/05 - Added support for aliasing/cancelling references
				//treeNode = parentNode.Nodes.Add("", refDisplayName);
				//if (refNode.Reference != null 
				//	|| !aliasManager.AreExcluded(refNode.ChildReferences))
			//	{
					//treeNode = parentNode.Nodes.Add("", refDisplayName);	
                // LZ-GSG 03/14/2013
                    if (refDisplayName == null)
                    {
                        treeNode = parentNode.Nodes.Add(refNode.DisplayNameResolved, refNode.DisplayNameResolved);
                    }
                    else
                    {
                        treeNode = parentNode.Nodes.Add(refNode.DisplayNameResolved, refDisplayName);
                    }

			//	}				
			}

			// MRS 1/17/05 - If a category node has no children
			// or all of it's children are excluded, then there will be no
			// node here. So bail. 
			if (treeNode == null) return;

			if ( refNode.IsDataReference &&  refNode.Reference != null )
			{
				treeNode.Override.NodeAppearance.FontData.Bold = DefaultableBoolean.True;
			}

			treeNode.Override.NodeAppearance.Image = FormulaUITypeEditor.GetOperandNodeImage(refNode);			
			
			// MRS BR00802
			// Moved down. Only do this if there are children
//			if ( refNode.Reference == formulaProvider.Reference )
//				treeNode.Override.NodeAppearance.ForeColor = SystemColors.GrayText;

			ReferenceNode[] refNodes = refNode.ChildReferences;

			if ( refNodes != null && refNodes.Length > 0 )
			{
				// MRS BR00802
				// Moved from above. Only do this if there are children
				if ( refNode.Reference == formulaProvider.Reference )
					treeNode.Override.NodeAppearance.ForeColor = SystemColors.GrayText;

				foreach (ReferenceNode childRefNode in refNodes )
				{
					FormulaUITypeEditor.AddOperandNode(treeNode, childRefNode, formulaProvider, aliasManager);
				}
			}
			else
			{
				// MRS BR00802
				// If there are no children, disable the node
				if ( refNode.Reference == formulaProvider.Reference )
					treeNode.Enabled = false;
			}
		}	
		
		#endregion AddOperandNode

		#region GetReferenceDisplayName
		internal static string GetReferenceDisplayName(ReferenceNode refNode, AliasManager aliasManager)
		{
            // LZ-GSG 03/14/2013
            return aliasManager.GetAliasByKey(refNode);
			//return refNode.DisplayNameResolved;			
		}
		#endregion GetReferenceDisplayName

		#region PopulateOperandsMenu

		internal static void PopulateOperandsMenu(PopupMenuTool menu, IFormulaBuilderInfo designInfo, IFormulaProvider formulaProvider, AliasManager aliasManager)
		{
			FormulaUITypeEditor.nextToolKey = 0;

			// MRS 1/17/05 - Added support for aliasing/cancelling references
			//ReferenceNode[] refNodes = designInfo.GetReferenceTree();
			ReferenceNode[] refNodes = aliasManager.refTree;
			
			menu.Tools.Clear();

            // MBS 12/21/06
            // Added support for conditional formatting to show the node at the top of the tree
            if (formulaProvider is IFormulaProviderEx)
            {
                ReferenceNode node = ((IFormulaProviderEx)formulaProvider).FormulaDialogReferenceNode;
                if (node != null)
                    FormulaUITypeEditor.AddOperandMenuItem(menu, node, formulaProvider, aliasManager);
            }

			foreach (ReferenceNode refNode in refNodes )
			{				
				FormulaUITypeEditor.AddOperandMenuItem(menu, refNode, formulaProvider, aliasManager);
			}
		}	

		#endregion PopulateOperandsMenu
	
		#region AddOperandMenuItem

		internal static int nextToolKey = 0;
		internal static void AddOperandMenuItem(PopupMenuTool parentMenu, ReferenceNode refNode, IFormulaProvider formulaProvider, AliasManager aliasManager)
		{
			//MRS 1/12/05 - Added support for aliasing/cancelling references.
			if ( aliasManager.IsExcluded(refNode) )
				return;

			ToolBase newTool =null;
			string toolKey = "Operand:" + FormulaUITypeEditor.nextToolKey.ToString();
			FormulaUITypeEditor.nextToolKey++;

			if ( refNode.ChildReferences != null && 
				refNode.ChildReferences.Length > 0 )
			{
				newTool = (ToolBase)new PopupMenuTool(toolKey);
			}
			else
			{
				// SSP 1/31/05 BR02097
				// Return if the item is not a data reference and doesn't have any data reference 
				// descendants. For example if there are no named references at all so the 
				// ChildReferences would be of length 0 and we would end up adding button tool for 
				// (All) item. Clicking on it causes null reference exception.
				// 
				if ( ! refNode.IsDataReference )
					return;

				newTool = (ToolBase)new ButtonTool(toolKey);
			}
			
			parentMenu.ToolbarsManager.Tools.Add(newTool);
			newTool.SharedProps.Caption = refNode.DisplayNameResolved;
			parentMenu.Tools.AddTool(toolKey);
            
			if ( refNode.IsDataReference && 
				refNode.Reference != null )				
			{
				newTool.SharedProps.AppearancesSmall.Appearance.FontData.Bold = DefaultableBoolean.True;				
				string relativeName = FormulaUITypeEditor.GetRelativeReferenceName(formulaProvider.Reference, refNode.Reference);
				newTool.SharedProps.Tag = relativeName;
			}

			newTool.SharedProps.AppearancesSmall.Appearance.Image = FormulaUITypeEditor.GetOperandNodeImage(refNode);			

			// MRS 1/11/05 - BR00802 
			// This is all wrong. Re-wrote it
//			// MRS BR00802
//			// Moved down. Only do this if there are children
////			if ( refNode.Reference == formulaProvider.Reference )
////				newTool.SharedProps.AppearancesSmall.Appearance.ForeColor = SystemColors.GrayText;
//			
//			if ( refNodes != null && refNodes.Length > 0 )
//			{
//				// MRS BR00802
//				// Moved from above. Only do this if there are children
//				if ( refNode.Reference == formulaProvider.Reference )
//					newTool.SharedProps.AppearancesSmall.Appearance.ForeColor = SystemColors.GrayText;
//
//				foreach (ReferenceNode childRefNode in refNodes )
//				{
//					FormulaUITypeEditor.AddOperandMenuItem((PopupMenuTool)newTool, childRefNode, formulaProvider);
//				}
//			}
//			else
//			{
//				// MRS BR00802
//				// If the menu has no children, disable it. 				
//				newTool.SharedProps.Enabled = false;
//			}

			// Is this the reference whose formula is being edited
			if ( refNode.Reference == formulaProvider.Reference )
			{
				//Is it a menu tool (does it have children)?
				if (newTool is PopupMenuTool)
				{
					// It's a menu tool, so we want to grey it out rather than disabling it. 
					newTool.SharedProps.AppearancesSmall.Appearance.ForeColor = SystemColors.GrayText;
				}
				else
				{
					// It's not a menu tool (there are no children) so 
					// just disable this guy on the menu. 
					newTool.SharedProps.Enabled = false;
				}
			}

			// Add the children
			if ( refNode.ChildReferences != null &&
				refNode.ChildReferences.Length > 0 )
			{				
				ReferenceNode[] refNodes = refNode.ChildReferences;

				foreach (ReferenceNode childRefNode in refNodes )
				{
					FormulaUITypeEditor.AddOperandMenuItem((PopupMenuTool)newTool, childRefNode, formulaProvider, aliasManager);
				}
			}
		}	
		
		#endregion AddOperandMenuItem
		
		#region GetOperandNodeImage

		private static object GetOperandNodeImage(ReferenceNode refNode)
		{
			switch (refNode.ReferenceNodeType)
			{
				case ReferenceNodeType.Control:	
					if ( refNode.Reference == null ||
						refNode.Reference.Context == null )
						return null;

					Image toolBoxImage = FormulaUITypeEditor.GetToolboxBitmap(refNode.Reference.Context.GetType());
					return toolBoxImage;
				
				case ReferenceNodeType.ControlsGroup:
					return 1;					
				case ReferenceNodeType.NamedReference:
					return 3;
				case ReferenceNodeType.NamedReferencesAllGroup:
					return 0;
				case ReferenceNodeType.NamedReferencesGroup:
					return 2;
				case ReferenceNodeType.NamedReferencesOtherGroup:
					return 5;
				case ReferenceNodeType.NamedReferencesUnassignedGroup:
					return 4;
			}

			return null;
		}
		#endregion GetOperandNodeImage

		#region IsReferenceSimple

		private static bool IsReferenceSimple(IFormulaProvider formulaProvider)
		{
			return (formulaProvider is NamedReference || formulaProvider is CalcSettings);
		}

		#endregion IsReferenceSimple

		#region GetRelativeReferenceName

		private static string GetRelativeReferenceName(IUltraCalcReference baseReference, IUltraCalcReference relativeReference)
		{
			if (baseReference == relativeReference )
				return baseReference.ElementName;

			string baseRefName = baseReference.AbsoluteName.ToLower();
			RefParser baseRefParser = new RefParser( baseRefName );

			string relativeRefName = relativeReference.AbsoluteName.ToLower();
			RefParser relativeRefParser = new RefParser( relativeRefName );
			
			if ( baseRefParser.TupleCount > 1 && 
				relativeRefParser.TupleCount > 1 &&
				baseReference.Parent == relativeReference.Parent )
				return relativeReference.ElementName;

			int shortestReferenceTupleCount = Math.Min(baseRefParser.TupleCount, relativeRefParser.TupleCount);

			int lastMatchingTupleIndex = -1;
			for ( int i = 0 ; i < shortestReferenceTupleCount; i++ )
			{
				if ( baseRefParser[i].Name == relativeRefParser[i].Name )
					lastMatchingTupleIndex = i;
				else
					break;
			}

			if ( lastMatchingTupleIndex == -1 )
				return relativeReference.AbsoluteName;

			System.Text.StringBuilder SB = new System.Text.StringBuilder();
			    
			for ( int i = 0; i<= lastMatchingTupleIndex; i++ )
			{
				if (i == 0)
					SB.Append("//");
				else
					SB.Append("/");

				SB.Append(baseRefParser[i]);
			}
			string commonAncestor = SB.ToString().ToLower();

			baseRefName = baseRefName.Replace(commonAncestor, String.Empty);
			if ( baseRefName.StartsWith("/") )
				baseRefName = baseRefName.Substring(1);

			relativeRefName = relativeRefName.Replace(commonAncestor, String.Empty);
			if ( relativeRefName.StartsWith("/") )
				relativeRefName = relativeRefName.Substring(1);

			SB.Length = 0;
			if ( baseRefName.Length >0 )
			{
				baseRefParser = new RefParser( baseRefName );
				for ( int i = 0; i < baseRefParser.TupleCount; i++ )
				{
					SB.Append("../");
				}
			}

			SB.Append (relativeRefName);

			return SB.ToString();			
		}

		#endregion GetRelativeReferenceName

		#region GetToolboxBitmap

		internal static Image GetToolboxBitmap( Type type )
		{
			System.Drawing.Image image = null;

			object[] attribs = type.GetCustomAttributes( typeof(ToolboxBitmapAttribute), true );

			if (attribs != null && attribs.Length > 0)
			{
				foreach( ToolboxBitmapAttribute tba in attribs )
				{
					image = tba.GetImage(type);
				}
			}
			else
			{
				// otherwise look in the file's resources
				string name = type.FullName + ".bmp";
				System.IO.Stream stream = type.Assembly.GetManifestResourceStream(name);

				if (stream != null)
					image = Bitmap.FromStream(stream);
			}
			
			if ( image == null ) return null;

			if (image is Bitmap)
				((Bitmap)image).MakeTransparent();

			return image;
		}

		#endregion GetToolboxBitmap

		#region PopulateFunctionsTree

		internal static void PopulateFunctionsTree(UltraTree tree, IFormulaBuilderInfo designInfo, AliasManager aliasManager)
		{			
			UltraTreeNode categoryNode = null;

			tree.Override.SelectionType = SelectType.SingleAutoDrag;
			tree.Override.Sort = SortType.Ascending;

			tree.Nodes.Clear();
            
			//MRS 1/12/05 - Make sure at least one Unassigned Function has not been filtered out
			UltraCalcFunction[] allFunctions = designInfo.GetAllFunctions();
			if (!aliasManager.AreExcluded(allFunctions))
			{
				UltraTreeNode allNode = tree.Nodes.Add("BIF:All", FormulaUITypeEditor.FunctionAllCategoryName );

				allNode.Override.NodeAppearance.Image = 0;
				FormulaUITypeEditor.AddFunctionNodes(allNode, allFunctions, aliasManager);
			}

			// MRS 9/15/04 UWC130
			// If there are no Unassigned functions, then don't add the menu
			UltraCalcFunction[] unassignedFunctions = designInfo.GetUnassignedFunctions();
			if ( unassignedFunctions.Length > 0 )
			{
				//MRS 1/12/05 - Make sure at least one Unassigned Function has not been filtered out
				if (!aliasManager.AreExcluded(unassignedFunctions))
				{
					UltraTreeNode unassignedNode = tree.Nodes.Add("BIF:Unassigned", FormulaUITypeEditor.FunctionUnassignedCategoryName);
					unassignedNode.Override.NodeAppearance.Image = 3;
					FormulaUITypeEditor.AddFunctionNodes(unassignedNode, unassignedFunctions, aliasManager);
				}
			}

			string[] categories = designInfo.GetUniqueFunctionCategories();
			foreach ( string category in categories )
			{
				if ( category != FormulaUITypeEditor.FunctionAllCategoryName && 
					category != FormulaUITypeEditor.FunctionUnassignedCategoryName)
				{
					//MRS 1/12/05 - Make sure at least one Unassigned Function has not been filtered out
					UltraCalcFunction[] categoryFunctions = designInfo.GetFunctions(category);
					if (!aliasManager.AreExcluded(categoryFunctions))
					{
						categoryNode = tree.Nodes.Add("BIF:" + category, category);
						categoryNode.Override.NodeAppearance.Image = 2;
						FormulaUITypeEditor.AddFunctionNodes(categoryNode, categoryFunctions, aliasManager);
					}
				}
			}			
		}

		#endregion PopulateFunctionsTree

		#region AddFunctionNodes

		internal static void AddFunctionNodes(UltraTreeNode categoryNode, UltraCalcFunction[] functions, AliasManager aliasManager)
		{
			foreach ( UltraCalcFunction function in functions)
			{	
				//MRS 1/12/05 - Added support for cancelling functions.
				if (!aliasManager.IsExcluded(function))
				{
					UltraTreeNode functionNode = categoryNode.Nodes.Add(null,function.Name);
					functionNode.Override.NodeAppearance.FontData.Bold = DefaultableBoolean.True;
					if (function.GetType().Assembly == typeof(UltraCalcManager).Assembly)  //!function.IsUserDefined)
						functionNode.Override.NodeAppearance.Image = 1;
					else
						functionNode.Override.NodeAppearance.Image = 4;

					functionNode.Override.ShowExpansionIndicator = ShowExpansionIndicator.Never;
					functionNode.Override.NodeDoubleClickAction = NodeDoubleClickAction.None;
					functionNode.Tag = function;
				}
			}
		}

		#endregion AddFunctionNodes

		#region PopulateFunctionsMenu

		internal static void PopulateFunctionsMenu(PopupMenuTool menu, IFormulaBuilderInfo designInfo, AliasManager aliasManager)
		{			
			PopupMenuTool categoryMenu = null;

			menu.Tools.Clear();
                        
			PopupMenuTool allMenu = new PopupMenuTool("BIF:All");
			allMenu.SharedProps.Caption = FormulaUITypeEditor.FunctionAllCategoryName;
			allMenu.SharedProps.AppearancesSmall.Appearance.Image = 0;
            menu.ToolbarsManager.Tools.Add(allMenu);
			menu.Tools.AddTool(allMenu.Key);			
			FormulaUITypeEditor.AddFunctionMenus(allMenu, designInfo.GetAllFunctions(), aliasManager);

			// MRS 9/15/04 UWC130
			// If there are no Unassigned functions, then don't add the menu
			UltraCalcFunction[] UnassignedFunctions = designInfo.GetUnassignedFunctions();
			if ( UnassignedFunctions.Length > 0 )
			{
				PopupMenuTool unassignedMenu = new PopupMenuTool("BIF:Unassigned");
				unassignedMenu.SharedProps.Caption = FormulaUITypeEditor.FunctionUnassignedCategoryName;
				unassignedMenu.SharedProps.AppearancesSmall.Appearance.Image = 3;
				menu.ToolbarsManager.Tools.Add(unassignedMenu);
				menu.Tools.AddTool(unassignedMenu.Key);			
				FormulaUITypeEditor.AddFunctionMenus(unassignedMenu, UnassignedFunctions, aliasManager);
			}

			string[] categories = designInfo.GetUniqueFunctionCategories();
			foreach ( string category in categories )
			{
				if ( category != FormulaUITypeEditor.FunctionAllCategoryName && 
					category != FormulaUITypeEditor.FunctionUnassignedCategoryName)
				{
					categoryMenu = new PopupMenuTool("BIF:" + category);
					categoryMenu.SharedProps.Caption = category;
					categoryMenu.SharedProps.AppearancesSmall.Appearance.Image = 2;
					menu.ToolbarsManager.Tools.Add(categoryMenu);
					menu.Tools.AddTool(categoryMenu.Key);			
					FormulaUITypeEditor.AddFunctionMenus(categoryMenu, designInfo.GetFunctions(category), aliasManager);
				}
			}			
		}

		#endregion PopulateFunctionsMenu

		#region AddFunctionMenus

		internal static void AddFunctionMenus(PopupMenuTool categoryMenu, UltraCalcFunction[] functions, AliasManager aliasManager)
		{
			foreach ( UltraCalcFunction function in functions)
			{	
				//MRS 1/12/05 - Added support for cancelling functions.
				if (!aliasManager.IsExcluded(function))
				{
					string toolKey = "Function:" + FormulaUITypeEditor.nextToolKey.ToString();
					FormulaUITypeEditor.nextToolKey++;

					ButtonTool functionButton = new ButtonTool(toolKey);
					functionButton.SharedProps.Caption = function.Name;				
					functionButton.SharedProps.AppearancesSmall.Appearance.FontData.Bold = DefaultableBoolean.True;
					if (function.GetType().Assembly == typeof(UltraCalcManager).Assembly) //!function.IsUserDefined)
						functionButton.SharedProps.AppearancesSmall.Appearance.Image = 1;
					else
						functionButton.SharedProps.AppearancesSmall.Appearance.Image = 4;

					categoryMenu.ToolbarsManager.Tools.Add(functionButton);
					categoryMenu.Tools.AddTool(functionButton.Key);
					functionButton.SharedProps.Tag = function;
				}
			}
		}

		#endregion AddFunctionMenus

		#endregion Private / Internal Methods

		#endregion Methods

		#region Overrides

		/// <summary>
        /// Gets the editor style used by the <see cref="EditValue(System.ComponentModel.ITypeDescriptorContext,System.IServiceProvider,System.Object)"/> method.
		/// </summary>
        /// <param name="context">An <see cref="System.ComponentModel.ITypeDescriptorContext"/> that can be used to gain additional context information.</param>
        /// <returns>A <see cref="System.Drawing.Design.UITypeEditorEditStyle"/> value that indicates the style of editor used by the <see cref="EditValue(System.ComponentModel.ITypeDescriptorContext,System.IServiceProvider,System.Object)"/> method.</returns>
		public override System.Drawing.Design.UITypeEditorEditStyle GetEditStyle(System.ComponentModel.ITypeDescriptorContext context)
		{
			return UITypeEditorEditStyle.Modal;
		}

		/// <summary>
        /// Edits the specified object's value using the editor style indicated by the <see cref="GetEditStyle(System.ComponentModel.ITypeDescriptorContext)"/> method.
		/// </summary>
        /// <param name="context">An <see cref="System.ComponentModel.ITypeDescriptorContext"/> that can be used to gain additional context information.</param>
        /// <param name="provider">An <see cref="System.IServiceProvider"/> that this editor can use to obtain services.</param>
        /// <param name="value">The object to edit.</param>
        /// <returns>The new value of the object. If the value of the object has not changed, this should return the same object it was passed.</returns>
		public override object EditValue(System.ComponentModel.ITypeDescriptorContext context, System.IServiceProvider provider, object value)
		{
			if ( context == null || 
				context.Instance == null )
				return null;
			
			this.formulaProvider = context.Instance as IFormulaProvider;
			if ( formulaProvider == null )
				throw new NotSupportedException(Shared.SR.GetString("LE_Exception_1"));

			if ( formulaProvider.CalcManager == null )
				throw new NotSupportedException(Shared.SR.GetString("LE_Exception_2"));

			this.designInfo = formulaProvider.CalcManager as IFormulaBuilderInfo;
			if ( formulaProvider.CalcManager == null )
				throw new NotSupportedException(Shared.SR.GetString("LE_Exception_3"));
			
//			CalcSettings calcSettings = formulaProvider as CalcSettings;
//			if ( calcSettings != null && 
//				( calcSettings.PropertyName == null || calcSettings.PropertyName.Length == 0 ) )
//				throw new NotSupportedException(Shared.SR.GetString("LE_Exception_4"));

			string originalFormula = formulaProvider.Formula;

			//MRS 1/10/05
			//this.formulaBuilder = new FormulaBuilderDialog(this);
			// MRS 1/21/05
			//this.formulaBuilder = new FormulaBuilderDialog(formulaProvider);
			// MRS 12/5/06 - BR18312
			//
			//this.formulaBuilder = new FormulaBuilderDialog(formulaProvider, true);
			this.formulaBuilder = new FormulaBuilderDialog(formulaProvider, false);
			
			IWindowsFormsEditorService edSvc = (IWindowsFormsEditorService)provider.GetService(typeof(IWindowsFormsEditorService));

			if (edSvc == null)
				return value;

			//this.VerifyDuplicateReferences();

			DialogResult result = edSvc.ShowDialog(formulaBuilder);
			if ( result == DialogResult.OK )
				return formulaBuilder.Formula;

			return originalFormula;
		}

		#endregion Overrides

	}
}

#region Copyright (c) 2001-2012 Infragistics, Inc. All Rights Reserved
/* ---------------------------------------------------------------------*
*                           Infragistics, Inc.                          *
*              Copyright (c) 2001-2012 All Rights reserved               *
*                                                                       *
*                                                                       *
* This file and its contents are protected by United States and         *
* International copyright laws.  Unauthorized reproduction and/or       *
* distribution of all or any portion of the code contained herein       *
* is strictly prohibited and will result in severe civil and criminal   *
* penalties.  Any violations of this copyright will be prosecuted       *
* to the fullest extent possible under law.                             *
*                                                                       *
* THE SOURCE CODE CONTAINED HEREIN AND IN RELATED FILES IS PROVIDED     *
* TO THE REGISTERED DEVELOPER FOR THE PURPOSES OF EDUCATION AND         *
* TROUBLESHOOTING. UNDER NO CIRCUMSTANCES MAY ANY PORTION OF THE SOURCE *
* CODE BE DISTRIBUTED, DISCLOSED OR OTHERWISE MADE AVAILABLE TO ANY     *
* THIRD PARTY WITHOUT THE EXPRESS WRITTEN CONSENT OF INFRAGISTICS, INC. *
*                                                                       *
* UNDER NO CIRCUMSTANCES MAY THE SOURCE CODE BE USED IN WHOLE OR IN     *
* PART, AS THE BASIS FOR CREATING A PRODUCT THAT PROVIDES THE SAME, OR  *
* SUBSTANTIALLY THE SAME, FUNCTIONALITY AS ANY INFRAGISTICS PRODUCT.    *
*                                                                       *
* THE REGISTERED DEVELOPER ACKNOWLEDGES THAT THIS SOURCE CODE           *
* CONTAINS VALUABLE AND PROPRIETARY TRADE SECRETS OF INFRAGISTICS,      *
* INC.  THE REGISTERED DEVELOPER AGREES TO EXPEND EVERY EFFORT TO       *
* INSURE ITS CONFIDENTIALITY.                                           *
*                                                                       *
* THE END USER LICENSE AGREEMENT (EULA) ACCOMPANYING THE PRODUCT        *
* PERMITS THE REGISTERED DEVELOPER TO REDISTRIBUTE THE PRODUCT IN       *
* EXECUTABLE FORM ONLY IN SUPPORT OF APPLICATIONS WRITTEN USING         *
* THE PRODUCT.  IT DOES NOT PROVIDE ANY RIGHTS REGARDING THE            *
* SOURCE CODE CONTAINED HEREIN.                                         *
*                                                                       *
* THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE.              *
* --------------------------------------------------------------------- *
*/
#endregion Copyright (c) 2001-2012 Infragistics, Inc. All Rights Reserved