using System;
using System.Windows.Forms;
using Infragistics.Win;
using Infragistics.Win.CalcEngine;
using Infragistics.Win.UltraWinTree;
using Infragistics.Win.UltraWinGrid;
using System.Drawing;
using System.Collections;
using System.Diagnostics;


namespace Infragistics.Win.UltraWinCalcManager.FormulaBuilder
{
	#region OperandTreeDragHelper

	internal class OperandTreeDragHelper 
	{
		private UltraTree tree = null;		
		internal bool isDragging = false;

		public OperandTreeDragHelper(UltraTree tree)
		{
			this.tree = tree;
			this.tree.SelectionDragStart += new System.EventHandler(this.operandsTree_SelectionDragStart);
			this.tree.BeforeSelect += new Infragistics.Win.UltraWinTree.BeforeNodeSelectEventHandler(this.operandsTree_BeforeSelect);
			this.tree.QueryContinueDrag += new System.Windows.Forms.QueryContinueDragEventHandler (this.operandsTree_QueryContinueDrag);
			this.tree.Disposed += new System.EventHandler(this.operandsTree_Disposed);
			
		}

		#region operandsTree_SelectionDragStart

		private void operandsTree_SelectionDragStart(object sender, System.EventArgs e)
		{
			UltraTree tree = sender as UltraTree;

			if ( tree.SelectedNodes.Count != 1 )
				return;

			ReferenceTreeNode node = tree.SelectedNodes[0] as ReferenceTreeNode;

			if ( node  == null )
				return;

			this.isDragging = true;
			tree.DoDragDrop( node.RelativeName, DragDropEffects.All);
			this.isDragging = false;
		
		}

		#endregion operandsTree_SelectionDragStart

		#region operandsTree_BeforeSelect

		private void operandsTree_BeforeSelect(object sender, Infragistics.Win.UltraWinTree.BeforeSelectEventArgs e)
		{
			if ( e.NewSelections.Count > 0 && 
				! (e.NewSelections[0] is ReferenceTreeNode ) )
				e.Cancel = true;
		}

		#endregion operandsTree_BeforeSelect

		#region operandsTree_QueryContinueDrag

		private void operandsTree_QueryContinueDrag(object sender, System.Windows.Forms.QueryContinueDragEventArgs e)
		{
			if ( e.EscapePressed )
			{
				e.Action = DragAction.Cancel;
				this.isDragging = false;
			}
		}

		#endregion operandsTree_QueryContinueDrag

		#region operandsTree_Disposed

		private void operandsTree_Disposed(object sender, System.EventArgs e)
		{
			this.tree.SelectionDragStart -= new System.EventHandler(this.operandsTree_SelectionDragStart);
			this.tree.BeforeSelect -= new Infragistics.Win.UltraWinTree.BeforeNodeSelectEventHandler(this.operandsTree_BeforeSelect);
			this.tree.QueryContinueDrag -= new System.Windows.Forms.QueryContinueDragEventHandler (this.operandsTree_QueryContinueDrag);			
		}

		#endregion operandsTree_Disposed
	}

	#endregion OperandTreeDragHelper

	#region FunctionTreeDragHelper

	internal class FunctionTreeDragHelper 
	{
		private UltraTree tree = null;
		internal bool isDragging = false;

		static public readonly string FunctionDragString = "This string indicates that a function is being dragged from the tree into the formula box.";
		
		public FunctionTreeDragHelper(UltraTree tree)
		{
			this.tree = tree;
			this.tree.SelectionDragStart += new System.EventHandler(this.functionsTree_SelectionDragStart);
			this.tree.BeforeSelect += new Infragistics.Win.UltraWinTree.BeforeNodeSelectEventHandler(this.functionsTree_BeforeSelect);
			this.tree.QueryContinueDrag += new System.Windows.Forms.QueryContinueDragEventHandler (this.functionsTree_QueryContinueDrag);
			this.tree.Disposed += new System.EventHandler(this.functionsTree_Disposed);
		}


		#region functionsTree_SelectionDragStart

		private void functionsTree_SelectionDragStart(object sender, System.EventArgs e)
		{
			UltraTree tree = sender as UltraTree;

			if ( tree.SelectedNodes.Count != 1 )
				return;

			UltraTreeNode node = tree.SelectedNodes[0] as UltraTreeNode;

			if ( node  == null )
				return;
			
			UltraCalcFunction function = node.Tag as UltraCalcFunction;
			if ( function == null )
				return;
			
			this.isDragging = true;
			tree.DoDragDrop(FunctionDragString, DragDropEffects.All);		
			this.isDragging = false;
		}

		#endregion functionsTree_SelectionDragStart

		#region functionsTree_BeforeSelect

		private void functionsTree_BeforeSelect(object sender, Infragistics.Win.UltraWinTree.BeforeSelectEventArgs e)
		{
			if ( e.NewSelections.Count > 0 && 
				! (e.NewSelections[0].Tag is UltraCalcFunction ) )
				e.Cancel = true;
		}

		#endregion functionsTree_BeforeSelect

		#region functionsTree_QueryContinueDrag

		private void functionsTree_QueryContinueDrag(object sender, System.Windows.Forms.QueryContinueDragEventArgs e)
		{			
			if ( e.EscapePressed )
				e.Action = DragAction.Cancel;
		}

		#endregion functionsTree_QueryContinueDrag

		#region functionsTree_Disposed

		private void functionsTree_Disposed(object sender, System.EventArgs e)
		{
			this.tree.SelectionDragStart -= new System.EventHandler(this.functionsTree_SelectionDragStart);
			this.tree.BeforeSelect -= new Infragistics.Win.UltraWinTree.BeforeNodeSelectEventHandler(this.functionsTree_BeforeSelect);
			this.tree.QueryContinueDrag -= new System.Windows.Forms.QueryContinueDragEventHandler (this.functionsTree_QueryContinueDrag);
		}

		#endregion functionsTree_Disposed
	}

	#endregion FunctionTreeDragHelper

	#region ReferenceTreeNode
	internal class ReferenceTreeNode : UltraTreeNode
	{
		public ReferenceTreeNode(string text, IUltraCalcReference reference, string relativeName) : base(null, text) 
		{
			this.reference = reference;
			this.relativeName = relativeName;
		}
        
		private IUltraCalcReference reference = null;
		private string relativeName = null;
		private bool isBaseReference = false;

		public IUltraCalcReference Reference 
		{
			get { return this.reference; }
			set { this.reference = value; }
		}

		public string RelativeName
		{
			get { return this.relativeName; }
			set { this.relativeName = value; }
		}

		public bool IsBaseReference
		{
			get { return this.isBaseReference; }
			set { this.isBaseReference = value; }
		}
	}
	#endregion ReferenceTreeNode

	#region TreeStateManager

	internal class TreeStateManager
	{		
		private ArrayList expandedNodeList = null;

		#region Constructor

		public TreeStateManager()
		{
		}

		#endregion Constructor

		#region Store

		public void Store(UltraTree tree)
		{
			if ( this.expandedNodeList != null )
				this.expandedNodeList.Clear();

			this.Store(tree, tree.Nodes);
		}	

		public void Store(UltraTree tree, UltraWinTree.TreeNodesCollection nodes)
		{			
			foreach (UltraTreeNode node in nodes)
			{
				if ( node.Expanded )
				{
					this.ExpandedNodeList.Add(node.FullPath);
					this.Store(tree, node.Nodes);
				}
			}
		}

		#endregion Store

		#region Restore

		public void ReStore(UltraTree tree)
		{
			UltraTreeNode node = null;

			if ( this.expandedNodeList == null  )
				return;

			string[] fullPaths = (string[])this.ExpandedNodeList.ToArray(typeof(string));
			foreach ( string fullPath in  fullPaths)
			{
				node = this.GetNodeFromFullPath(tree, fullPath);
				if ( node != null )
				{
					node.Expanded = true;
				}
			}
		}	

		#region GetNodeFromFullPath

		private UltraTreeNode GetNodeFromFullPath(UltraTree tree, string fullPath )
		{
			string[] parts = fullPath.Split(tree.PathSeparator.ToCharArray());
			TreeNodesCollection nodes = tree.Nodes;
			UltraTreeNode node = null;

			foreach ( string part in parts )
			{
				if ( nodes.Count == 0 )
					return null;

				node = this.GetNodeFromText(part, nodes);
				if ( node == null )
					break;

				nodes = node.Nodes;
			}

			return node;
		}

		#endregion GetNodeFromFullPath
		
		#region GetNodeFromText

		private UltraTreeNode GetNodeFromText( string Text, TreeNodesCollection nodes )
		{
			foreach ( UltraTreeNode node in nodes )
			{
				if ( node.Text == Text )
					return node;
			}
			
			return null;
		}

		#endregion GetNodeFromText	

		#endregion Restore

		#region ExpandedNodeList Property

		public ArrayList ExpandedNodeList
		{
			get 
			{ 
				if ( this.expandedNodeList == null )
					this.expandedNodeList = new ArrayList();

				return this.expandedNodeList;			
			}        
		}

		#endregion ExpandedNodeList Property
	}

	#endregion TreeStateManager

	//MRS 1/11/05 - Added support for aliasing/cancelling references.
	#region AliasManager
	
	internal class AliasManager 
	{
		#region Private Members	
		
		private Hashtable referenceExcludeTable = null;
		private Hashtable functionExcludeTable = null;
		internal ReferenceNode[] refTree = null;
			
		#endregion Private Members

		#region Constructor
		internal AliasManager(ReferenceNode[] refTree) 
		{
			this.refTree = refTree;
		}
		#endregion Constructor

		#region Properties

		#region ReferenceExcludeTable
		internal Hashtable ReferenceExcludeTable
		{
			get
			{
				if (this.referenceExcludeTable == null)
					this.referenceExcludeTable = new Hashtable();

				return this.referenceExcludeTable;
			}
		}
		#endregion ReferenceExcludeTable

		#region FunctionExcludeTable
		internal Hashtable FunctionExcludeTable
		{
			get
			{
				if (this.functionExcludeTable == null)
					this.functionExcludeTable = new Hashtable();

				return this.functionExcludeTable;
			}
		}
		#endregion FunctionExcludeTable

		#endregion Properties		

		#region Methods

        // LZ-GSG 03/14/2013
        internal string GetAliasByKey(object Key)
        {
            if (this.referenceExcludeTable == null) return null;
            if (this.referenceExcludeTable.ContainsKey(Key))
            {
                return this.referenceExcludeTable[Key] as string;
            }
            else return null;
        }


		#region AddExclude
        // LZ-GSG 03/14/2013
		internal void AddExclude(ReferenceNode refNode, OperandInitializingEventArgs args)
		{
			this.ReferenceExcludeTable.Add(refNode, args.OperandAlias);
            
		}

		internal void AddExclude(UltraCalcFunction function)
		{
			this.FunctionExcludeTable.Add(function, null);
		}		
		#endregion AddExclude

		#region IsExcluded
		internal bool IsExcluded(ReferenceNode refNode)
		{
			return (this.ReferenceExcludeTable.ContainsKey(refNode));
		}

		internal bool IsExcluded(UltraCalcFunction function)
		{
			return (this.FunctionExcludeTable.ContainsKey(function));
		}

		internal bool AreExcluded(UltraCalcFunction[] functions)
		{
			foreach (UltraCalcFunction function in functions)
				if (!this.IsExcluded(function))
					return false;

			return true;
		}

		internal bool AreExcluded(ReferenceNode[] refNodes)
		{
			foreach (ReferenceNode refNode in refNodes)
				if (!this.IsExcluded(refNode))
					return false;

			return true;
		}
		#endregion IsExcluded

		#endregion Methods			
	}
	#endregion AliasManager

}


#region Copyright (c) 2001-2012 Infragistics, Inc. All Rights Reserved
/* ---------------------------------------------------------------------*
*                           Infragistics, Inc.                          *
*              Copyright (c) 2001-2012 All Rights reserved               *
*                                                                       *
*                                                                       *
* This file and its contents are protected by United States and         *
* International copyright laws.  Unauthorized reproduction and/or       *
* distribution of all or any portion of the code contained herein       *
* is strictly prohibited and will result in severe civil and criminal   *
* penalties.  Any violations of this copyright will be prosecuted       *
* to the fullest extent possible under law.                             *
*                                                                       *
* THE SOURCE CODE CONTAINED HEREIN AND IN RELATED FILES IS PROVIDED     *
* TO THE REGISTERED DEVELOPER FOR THE PURPOSES OF EDUCATION AND         *
* TROUBLESHOOTING. UNDER NO CIRCUMSTANCES MAY ANY PORTION OF THE SOURCE *
* CODE BE DISTRIBUTED, DISCLOSED OR OTHERWISE MADE AVAILABLE TO ANY     *
* THIRD PARTY WITHOUT THE EXPRESS WRITTEN CONSENT OF INFRAGISTICS, INC. *
*                                                                       *
* UNDER NO CIRCUMSTANCES MAY THE SOURCE CODE BE USED IN WHOLE OR IN     *
* PART, AS THE BASIS FOR CREATING A PRODUCT THAT PROVIDES THE SAME, OR  *
* SUBSTANTIALLY THE SAME, FUNCTIONALITY AS ANY INFRAGISTICS PRODUCT.    *
*                                                                       *
* THE REGISTERED DEVELOPER ACKNOWLEDGES THAT THIS SOURCE CODE           *
* CONTAINS VALUABLE AND PROPRIETARY TRADE SECRETS OF INFRAGISTICS,      *
* INC.  THE REGISTERED DEVELOPER AGREES TO EXPEND EVERY EFFORT TO       *
* INSURE ITS CONFIDENTIALITY.                                           *
*                                                                       *
* THE END USER LICENSE AGREEMENT (EULA) ACCOMPANYING THE PRODUCT        *
* PERMITS THE REGISTERED DEVELOPER TO REDISTRIBUTE THE PRODUCT IN       *
* EXECUTABLE FORM ONLY IN SUPPORT OF APPLICATIONS WRITTEN USING         *
* THE PRODUCT.  IT DOES NOT PROVIDE ANY RIGHTS REGARDING THE            *
* SOURCE CODE CONTAINED HEREIN.                                         *
*                                                                       *
* THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE.              *
* --------------------------------------------------------------------- *
*/
#endregion Copyright (c) 2001-2012 Infragistics, Inc. All Rights Reserved